% Clear all
clear all; close all; clc
% 
fprintf(1,'_____________________________________________________\n');
fprintf(1,'IE309: Digital Signal Processing with FPGA.\n');
fprintf(1,'Using CORDIC for phase and magnitude computation \n');
fprintf(1,'Diego Barragan Guerrero\n');
fprintf(1,'_____________________________________________________\n');

% Iterations
num_iter = 6;

% Arctangente table
atan_tabela=zeros(1,num_iter-1);
for i=0:num_iter-1
    atan_tabela(i+1) = atan(2^-i) * 180/pi;
end

% Input angles (module =< 1)
fase=30;
Phase=fase*pi/180;
I = cos(Phase);
Q = sin(Phase);

% Rotate input vectors in pi/2 in order to lead to I and IV quadrant
if Q>0 % Move to I or IV quadrant
    Ip = +Q;
    Qp = -I;
    Z0 = +90;
else    % Move to I or IV quadrant
    Ip = -Q;
    Qp = +I;
    Z0 = -90;
end
I = Ip;
Q = Qp;

% Iterations
for iter=1:num_iter 
    L = iter-1;
    if Q>0%Y(MSB)= signal bit
        Ip = I + (2^-L)*Q; %In the FPGA, the multiply by 2^-L is a displacement to right of the bits
        Qp = Q - (2^-L)*I; %In the FPGA, the multiply by 2^-L is a displacement to right of the bits       
        Z0 = Z0 + atan_tabela(iter);
    else
        Ip = I - (2^-L)*Q; %In the FPGA, the multiply by 2^-L is a displacement to right of the bits
        Qp = Q + (2^-L)*I; %In the FPGA, the multiply by 2^-L is a displacement to right of the bits
        Z0 = Z0 - atan_tabela(iter);
    end
    fprintf(1,'Iteration %d: Z = %3.2f com Q=%3.2f \n', L,Z0,Q) %#ok<PRTCAL>
    I = Ip;
    Q = Qp;    
end
% Show results
fprintf(1,'_____________________________________________________\n');
fprintf(1,'Phase        = %3.2f graus\n', Phase*180/pi);
fprintf(1,'Phase Cordic = %3.2f graus\n', Z0);
fprintf(1,'_____________________________________________________\n');

% References:
% Simple Phase Estimator:
% http://luminouslogic.com/dsp-simple-phase-estimation-approximation-cordic-matlab.htm
% Cordic: 
% http://pt.wikipedia.org/wiki/CORDIC
% Using CORDIC for phase and magnitude computation:
% www.dsplog.com